@@ -49,6 +49,8 @@ G_DEFINE_TYPE (GUPnPService,
 struct _GUPnPServicePrivate {
         GUPnPRootDevice *root_device;
 
+        SoupSession     *session;
+
         guint            notify_available_id;
 
         GHashTable      *subscriptions;
@@ -96,15 +98,41 @@ typedef struct {
                                            subscription */
 } SubscriptionData;
 
+static SoupSession *
+gupnp_service_get_session (GUPnPService *service)
+{
+        if (! service->priv->session) {
+                GUPnPContext *context =
+                  gupnp_service_info_get_context (GUPNP_SERVICE_INFO (service));
+
+                /* Create a dedicated session for this service to
+                 * ensure that notifications are sent in the proper
+                 * order. The session from GUPnPContext may use
+                 * multiple connections.
+                 */
+                service->priv->session = soup_session_async_new_with_options
+                  (SOUP_SESSION_IDLE_TIMEOUT, 60,
+                   SOUP_SESSION_ASYNC_CONTEXT,
+                   gssdp_client_get_main_context (GSSDP_CLIENT (context)),
+                   SOUP_SESSION_MAX_CONNS_PER_HOST, 1,
+                   NULL);
+
+                if (g_getenv ("GUPNP_DEBUG")) {
+                        SoupLogger *logger;
+                        logger = soup_logger_new (SOUP_LOGGER_LOG_BODY, -1);
+                        soup_logger_attach (logger, service->priv->session);
+                }
+        }
+
+        return service->priv->session;
+}
+
 static void
 subscription_data_free (SubscriptionData *data)
 {
-        GUPnPContext *context;
         SoupSession *session;
 
-        context = gupnp_service_info_get_context
-                        (GUPNP_SERVICE_INFO (data->service));
-        session = gupnp_context_get_session (context);
+        session = gupnp_service_get_session (data->service);
 
         /* Cancel pending messages */
         while (data->pending_messages) {
@@ -559,19 +587,19 @@ gupnp_service_action_return_error (GUPnPServiceAction *action,
 }
 
 static void
-gupnp_service_init (GUPnPService *device)
+gupnp_service_init (GUPnPService *service)
 {
-        device->priv = G_TYPE_INSTANCE_GET_PRIVATE (device,
-                                                    GUPNP_TYPE_SERVICE,
-                                                    GUPnPServicePrivate);
+        service->priv = G_TYPE_INSTANCE_GET_PRIVATE (service,
+                                                     GUPNP_TYPE_SERVICE,
+                                                     GUPnPServicePrivate);
 
-        device->priv->subscriptions =
+        service->priv->subscriptions =
                 g_hash_table_new_full (g_str_hash,
                                        g_str_equal,
                                        NULL,
                                        (GDestroyNotify) subscription_data_free);
 
-        device->priv->notify_queue = g_queue_new ();
+        service->priv->notify_queue = g_queue_new ();
 }
 
 /* Generate a new action response node for @action_name */
@@ -704,8 +732,8 @@ control_server_handler (SoupServer        *server,
         *end = '\0';
 
         /* Parse action_node */
-        doc = xmlParseMemory (msg->request_body->data,
-                              msg->request_body->length);
+        doc = xmlRecoverMemory (msg->request_body->data,
+                                msg->request_body->length);
         action_node = xml_util_get_element ((xmlNode *) doc,
                                             "Envelope",
                                             "Body",
@@ -1212,8 +1240,10 @@ gupnp_service_set_property (GObject      *object,
 
         switch (property_id) {
         case PROP_ROOT_DEVICE:
-                service->priv->root_device =
-                        g_object_ref (g_value_get_object (value));
+                service->priv->root_device = g_value_get_object (value);
+                g_object_add_weak_pointer
+                        (G_OBJECT (service->priv->root_device),
+                         (gpointer *) &service->priv->root_device);
 
                 service->priv->notify_available_id =
                         g_signal_connect_object (service->priv->root_device,
@@ -1267,7 +1297,10 @@ gupnp_service_dispose (GObject *object)
                                  service->priv->notify_available_id);
                 }
 
-                g_object_unref (service->priv->root_device);
+                g_object_remove_weak_pointer
+                        (G_OBJECT (service->priv->root_device),
+                         (gpointer *) &service->priv->root_device);
+
                 service->priv->root_device = NULL;
         }
 
@@ -1305,6 +1338,11 @@ gupnp_service_finalize (GObject *object)
 
         g_queue_free (service->priv->notify_queue);
 
+        if (service->priv->session) {
+                g_object_unref (service->priv->session);
+                service->priv->session = NULL;
+        }
+
         /* Call super */
         object_class = G_OBJECT_CLASS (gupnp_service_parent_class);
         object_class->finalize (object);
@@ -1511,7 +1549,6 @@ notify_got_response (SoupSession *session,
                 /* Other failure: Try next callback or signal failure. */
                 if (data->callbacks->next) {
                         SoupURI *uri;
-                        GUPnPContext *context;
                         SoupSession *session;
 
                         /* Call next callback */
@@ -1525,18 +1562,17 @@ notify_got_response (SoupSession *session,
                         data->pending_messages = 
                                 g_list_prepend (data->pending_messages, msg);
 
-                        context = gupnp_service_info_get_context
-                                        (GUPNP_SERVICE_INFO (data->service));
-                        session = gupnp_context_get_session (context);
+                        session = gupnp_service_get_session (data->service);
 
                         soup_session_requeue_message (session, msg);
                 } else {
                         /* Emit 'notify-failed' signal */
                         GError *error;
 
-                        error = g_error_new (GUPNP_EVENTING_ERROR,
-                                             GUPNP_EVENTING_ERROR_NOTIFY_FAILED,
-                                             "%s", msg->reason_phrase);
+                        error = g_error_new_literal
+                                        (GUPNP_EVENTING_ERROR,
+                                         GUPNP_EVENTING_ERROR_NOTIFY_FAILED,
+                                         msg->reason_phrase);
 
                         g_signal_emit (data->service,
                                        signals[NOTIFY_FAILED],
@@ -1562,7 +1598,6 @@ notify_subscriber (gpointer key,
         const char *property_set;
         char *tmp;
         SoupMessage *msg;
-        GUPnPContext *context;
         SoupSession *session;
 
         data = value;
@@ -1603,7 +1638,7 @@ notify_subscriber (gpointer key,
 
         /* Add body */
         soup_message_set_request (msg,
-                                  "text/xml",
+                                  "text/xml; charset=utf-8",
                                   SOUP_MEMORY_TAKE,
                                   g_strdup (property_set),
                                   strlen (property_set));
@@ -1611,9 +1646,7 @@ notify_subscriber (gpointer key,
         /* Queue */
         data->pending_messages = g_list_prepend (data->pending_messages, msg);
 
-        context = gupnp_service_info_get_context
-                        (GUPNP_SERVICE_INFO (data->service));
-        session = gupnp_context_get_session (context);
+        session = gupnp_service_get_session (data->service);
 
         soup_session_queue_message (session,
                                     msg,
